import pytest
from pyhamtools.locator import locator_to_latlong
from pyhamtools.consts import LookupConventions as const

class Test_locator_to_latlong():

    def test_locator_to_latlong_edge_cases(self):
        latitude, longitude = locator_to_latlong("AA00AA")
        assert abs(latitude + 89.97916) < 0.00001
        assert abs(longitude +179.95833) < 0.0001

        latitude, longitude = locator_to_latlong("RR99XX")
        assert abs(latitude - 89.97916) < 0.00001
        assert abs(longitude - 179.9583) < 0.0001

    def test_locator_to_latlong_normal_case(self):

        latitude, longitude = locator_to_latlong("JN48QM")
        assert abs(latitude - 48.52083) < 0.00001
        assert abs(longitude - 9.3750000) < 0.0001

        latitude, longitude = locator_to_latlong("JN48")
        assert abs(latitude - 48.5) < 0.001
        assert abs(longitude - 9.000) < 0.001

    def test_locator_to_latlong_mixed_signs(self):

        latitude, longitude = locator_to_latlong("Jn48qM")
        assert abs(latitude - 48.52083) < 0.00001
        assert abs(longitude - 9.3750000) < 0.0001


    def test_locator_to_latlong_wrong_amount_of_characters(self):

        with pytest.raises(ValueError):
            latitude, longitude = locator_to_latlong("J")

        with pytest.raises(ValueError):
            latitude, longitude = locator_to_latlong("JN")

        with pytest.raises(ValueError):
            latitude, longitude = locator_to_latlong("JN4")

        with pytest.raises(ValueError):
            latitude, longitude = locator_to_latlong("JN8Q")

    def test_locator_to_latlong_invalid_characters(self):

        with pytest.raises(ValueError):
            latitude, longitude = locator_to_latlong("21XM99")

        with pytest.raises(ValueError):
            latitude, longitude = locator_to_latlong("****")

    def test_locator_to_latlong_out_of_boundry(self):

        with pytest.raises(ValueError):
            latitude, longitude = locator_to_latlong("RR99XY")



import os
import sys
import datetime

from past.builtins import execfile
from future.utils import iteritems
import pytest

from pyhamtools.qsl import get_lotw_users

if sys.version_info.major == 3:
    unicode = str

test_dir = os.path.dirname(os.path.abspath(__file__))
fix_dir = os.path.join(test_dir, 'fixtures')

class Test_lotw_methods:

    def test_check_content_with_mocked_http_server(self, httpserver):
        httpserver.serve_content(open(os.path.join(fix_dir, 'lotw-user-activity.csv')).read())

        namespace = {}
        execfile(os.path.join(fix_dir,"lotw_fixture.py"), namespace)
        assert get_lotw_users(url=httpserver.url) == namespace['lotw_fixture']

    def test_download_lotw_list_and_check_types(self):

        data = get_lotw_users()
        assert isinstance(data, dict)
        for key, value in iteritems(data):
            assert isinstance(key, unicode)
            assert isinstance(value, datetime.datetime )
        assert len(data) > 1000

    def test_with_invalid_url(self):
        with pytest.raises(IOError):
            get_lotw_users(url="https://lotw.arrl.org/lotw-user-activity_FAKE.csv")

    def test_with_more_than_10_invalid_dates(self, httpserver):
        httpserver.serve_content(open(os.path.join(fix_dir, 'lotw_data_with_errors.html')).read())

        with pytest.raises(ValueError):
            get_lotw_users(url=httpserver.url)





import pytest
from pyhamtools.locator import latlong_to_locator
from pyhamtools.consts import LookupConventions as const

class Test_latlong_to_locator():

    def test_latlong_to_locator_edge_cases(self):
        assert latlong_to_locator(-89.97916, -179.95833) == "AA00AA"
        assert latlong_to_locator(89.97916, 179.9583) == "RR99XX"

    def test_latlong_to_locator_normal_case(self):

        assert latlong_to_locator(48.52083, 9.3750000) == "JN48QM"
        assert latlong_to_locator(48.5, 9.0) == "JN48MM" #center of the square

    def test_latlong_to_locator_invalid_characters(self):

        # throws ValueError in Python2 and TypeError in Python3
        with pytest.raises(Exception):
            latlong_to_locator("JN48QM", "test")

        # throws ValueError in Python2 and TypeError in Python3
        with pytest.raises(Exception):
            latlong_to_locator("", "")

    def test_latlong_to_locator_out_of_boundry(self):

        with pytest.raises(ValueError):
            latlong_to_locator(-90, -180)

        with pytest.raises(ValueError):
            latlong_to_locator(90, 180)

        with pytest.raises(ValueError):
            latlong_to_locator(10000, 120000)



from datetime import datetime, timedelta

import pytest
import pytz

from pyhamtools.locator import calculate_sunrise_sunset

UTC = pytz.UTC

class Test_calculate_sunrise_sunset_normal_case():

    def test_calculate_sunrise_sunset(self):

        time_margin = timedelta(minutes=1)
        locator = "JN48QM"

        test_time =  datetime(year=2014, month=1, day=1, tzinfo=UTC)
        result_JN48QM_1_1_2014_evening_dawn = datetime(2014, 1, 1, 15, 38, tzinfo=UTC)
        result_JN48QM_1_1_2014_morning_dawn = datetime(2014, 1, 1, 6, 36,  tzinfo=UTC)
        result_JN48QM_1_1_2014_sunrise = datetime(2014, 1, 1, 7, 14, tzinfo=UTC)
        result_JN48QM_1_1_2014_sunset = datetime(2014, 1, 1, 16, 15, 23, 31016, tzinfo=UTC)

        assert calculate_sunrise_sunset(locator, test_time)['morning_dawn'] - result_JN48QM_1_1_2014_morning_dawn < time_margin
        assert calculate_sunrise_sunset(locator, test_time)['evening_dawn'] - result_JN48QM_1_1_2014_evening_dawn < time_margin
        assert calculate_sunrise_sunset(locator, test_time)['sunset'] - result_JN48QM_1_1_2014_sunset < time_margin
        assert calculate_sunrise_sunset(locator, test_time)['sunrise'] - result_JN48QM_1_1_2014_sunrise < time_margin

    def test_calculate_distance_edge_case(self):

        time_margin = timedelta(minutes=1)
        locator = "AA00AA"
        # no sunrise or sunset at southpol during arctic summer

        test_time =  datetime(year=2014, month=1, day=1, tzinfo=UTC)
        result_AA00AA_1_1_2014_evening_dawn = datetime(2014, 1, 1, 15, 38, tzinfo=UTC)
        result_AA00AA_1_1_2014_morning_dawn = datetime(2014, 1, 1, 6, 36,  tzinfo=UTC)
        result_AA00AA_1_1_2014_sunrise = datetime(2014, 1, 1, 7, 14, tzinfo=UTC)
        result_AA00AA_1_1_2014_sunset = datetime(2014, 1, 1, 16, 15, 23, 31016, tzinfo=UTC)

        assert calculate_sunrise_sunset(locator, test_time)['morning_dawn'] == None
        assert calculate_sunrise_sunset(locator, test_time)['evening_dawn'] == None
        assert calculate_sunrise_sunset(locator, test_time)['sunset'] == None
        assert calculate_sunrise_sunset(locator, test_time)['sunrise'] == None

    def test_calculate_distance_invalid_inputs(self):

        with pytest.raises(ValueError):
            calculate_sunrise_sunset("", "")

        with pytest.raises(ValueError):
            calculate_sunrise_sunset("JN48QM", "")

        with pytest.raises(ValueError):
            calculate_sunrise_sunset("JN48", 55)

        with pytest.raises(AttributeError):
            calculate_sunrise_sunset(33, datetime.now())
